@@ -85,47 +85,40 @@ SKAT_Scale_Genotypes= function(obj.res, Z1, Z2, weights.beta=c(1,25), weights.be
 
 
 
-#
-# x is either y or SKAT_NULL_Model 
-#
-SKAT_CommonRare.SSD.OneSet = function(SSD.INFO, SetID, obj, ...){
-	
-	id1<-which(SSD.INFO$SetInfo$SetID == SetID)
-	if(length(id1) == 0){
-		MSG<-sprintf("Error: cannot find set id [%s] from SSD!", SetID)
-		stop(MSG)
-	}	
-	Set_Index<-SSD.INFO$SetInfo$SetIndex[id1]
-
-	Z<-Get_Genotypes_SSD(SSD.INFO, Set_Index)
-	re<-SKAT_CommonRare(Z, obj, ...)
-	
-	return(re)
+SKAT_CommonRare.SSD.OneSet = function(SSD.INFO, SetID, obj, ..., obj.SNPWeight=NULL){
+  
+  id1<-which(SSD.INFO$SetInfo$SetID == SetID)
+  if(length(id1) == 0){
+    MSG<-sprintf("Error: cannot find set id [%s] from SSD!", SetID)
+    stop(MSG)
+  }	
+  SetIndex<-SSD.INFO$SetInfo$SetIndex[id1]
+  
+  re = SKAT_CommonRare.SSD.OneSet_SetIndex(SSD.INFO, SetIndex, obj, ..., obj.SNPWeight=obj.SNPWeight)
+  
+  return(re)
 }
 
-#
-# x is either y or SKAT_NULL_Model 
-#
-SKAT_CommonRare.SSD.OneSet_SetIndex = function(SSD.INFO, SetIndex, obj, ...){
-	
-	id1<-which(SSD.INFO$SetInfo$SetIndex == SetIndex)
-	if(length(id1) == 0){
-		MSG<-sprintf("Error: cannot find set index [%d] from SSD!", SetIndex)
-		stop(MSG)
-	}	
-	SetID<-SSD.INFO$SetInfo$SetID[id1]
-
-
-	Z<-Get_Genotypes_SSD(SSD.INFO, SetIndex)
-	re<-SKAT_CommonRare(Z, obj, ...)
-	return(re)
+SKAT_CommonRare.SSD.OneSet_SetIndex = function(SSD.INFO, SetIndex, obj, ..., obj.SNPWeight=NULL){
+  
+  re1 = SKAT.SSD.GetSNP_Weight(SSD.INFO, SetIndex, obj.SNPWeight=obj.SNPWeight)
+  SetID = SSD.INFO$SetInfo$SetID[SetIndex]
+  if(!re1$Is.weights){
+    re<-SKAT_CommonRare(re1$Z, obj, ...)
+  } else {
+    
+    re<-SKAT_CommonRare(re1$Z, obj, weights=re1$weights, ...)
+  }
+  
+  return(re)
+  
 }
 
 
 #
 # Only SKAT_Null_Model obj can be used
 #
-SKAT_CommonRare.SSD.All = function(SSD.INFO, obj, ...){
+SKAT_CommonRare.SSD.All = function(SSD.INFO, obj, ..., obj.SNPWeight=NULL){
 	
 	N.Set<-SSD.INFO$nSets
 	OUT.Pvalue<-rep(NA,N.Set)
@@ -134,6 +127,7 @@ SKAT_CommonRare.SSD.All = function(SSD.INFO, obj, ...){
 	OUT.Error<-rep(-1,N.Set)
 	OUT.Pvalue.Resampling<-NULL
 	OUT.Q<-rep(NA,N.Set)
+	OUT.snp.mac<-list()
 
 	OUT.nRare<-rep(NA,N.Set)
 	OUT.nCommon<-rep(NA,N.Set)
@@ -141,6 +135,7 @@ SKAT_CommonRare.SSD.All = function(SSD.INFO, obj, ...){
 	Is.Resampling = FALSE
 	n.Resampling = 0
 	
+	
 	if(Check_Class(obj, "SKAT_NULL_Model")){
 		if(obj$n.Resampling > 0){
 			Is.Resampling = TRUE
@@ -160,35 +155,15 @@ SKAT_CommonRare.SSD.All = function(SSD.INFO, obj, ...){
 	pb <- txtProgressBar(min=0, max=N.Set, style=3)
 	for(i in 1:N.Set){
 		Is.Error<-TRUE
-		try1<-try(Get_Genotypes_SSD(SSD.INFO, i),silent = TRUE)
-		if(!Is_TryError(try1)){
-			Z<-try1
-			Is.Error<-FALSE
-			
-			
+		try1<-try(SKAT_CommonRare.SSD.OneSet_SetIndex(SSD.INFO, i, obj, ..., obj.SNPWeight=obj.SNPWeight) ,silent = TRUE)
+		if(Is_TryError(try1)){
+		  
+		  err.msg<-geterrmessage()
+		  msg<-sprintf("Error to run SKATBinary for %s: %s",SSD.INFO$SetInfo$SetID[i], err.msg)
+		  warning(msg,call.=FALSE)
+		  
 		} else {
-			err.msg<-geterrmessage()
-			msg<-sprintf("Error to get genotypes of %s: %s",SSD.INFO$SetInfo$SetID[i], err.msg)
-			warning(msg,call.=FALSE)
-		}
-	
-		if(!Is.Error){
-			Is.Error<-TRUE
-			try2<-try(SKAT_CommonRare(Z,obj, ...),silent = TRUE)
-			
-			if(!Is_TryError(try2)){
-				re<-try2
-				Is.Error<-FALSE
-			} else {
-
-				err.msg<-geterrmessage()
-				msg<-sprintf("Error to run SKAT_CommonRare for %s: %s",SSD.INFO$SetInfo$SetID[i], err.msg)
-				warning(msg,call.=FALSE)
-			}
-		}
-		
-		if(!Is.Error){
-
+      re<-try1
 			OUT.Pvalue[i]<-re$p.value
 			OUT.Marker[i]<-re$param$n.marker
 			OUT.Marker.Test[i]<-re$param$n.marker.test
@@ -198,6 +173,8 @@ SKAT_CommonRare.SSD.All = function(SSD.INFO, obj, ...){
 			if(Is.Resampling){
 				OUT.Pvalue.Resampling[i,]<-re$p.value.resampling
 			}
+			SetID<-SSD.INFO$SetInfo$SetID[i]
+			OUT.snp.mac[[SetID]]<-re$test.snp.mac
 		}
 		#if(floor(i/100)*100 == i){
 		#	cat("\r", i, "/", N.Set, "were done");
@@ -209,7 +186,7 @@ SKAT_CommonRare.SSD.All = function(SSD.INFO, obj, ...){
 	close(pb)	
 	out.tbl<-data.frame(SetID=SSD.INFO$SetInfo$SetID, P.value=OUT.Pvalue, Q=OUT.Q
 	, N.Marker.All=OUT.Marker, N.Marker.Test=OUT.Marker.Test, N.Marker.Rare=OUT.nRare, N.Marker.Common=OUT.nCommon)
-	re<-list(results=out.tbl,P.value.Resampling=OUT.Pvalue.Resampling)
+	re<-list(results=out.tbl,P.value.Resampling=OUT.Pvalue.Resampling,OUT.snp.mac=OUT.snp.mac)
 	class(re)<-"SKAT_SSD_ALL"
 
 	return(re)	
@@ -362,6 +339,7 @@ r.corr.rare=0, r.corr.common=0, CommonRare_Cutoff=NULL, test.type="Joint", is_do
 	re$n.common = n.common
 	re$test.type= test.type
 	re$Cutoff = CommonRare_Cutoff
+	re$test.snp.mac<-SingleSNP_INFO(out$Z.test)	
 	return(re)
 
 }
